* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS LISTING;

***********************************************************************************;
*******   MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR CHAPTER 6 EXAMPLE     *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=,FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested model;
* CovMore =  Name of ODS CovParms table for comparison model;
%MACRO PseudoR2(NCov=,CovFewer=,CovMore=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
     THEN DELETE; RUN;
TITLE9 "PsuedoR2 (% Reduction) for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND PseudoR2;

***********************************************************************************;
*******             BEGIN DATA MANIPULATION OF CHAPTER 6 EXAMPLE            *******;
*******               CHANGE "filesave" TO YOUR DIRECTORY                   *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
%LET filesave=C:\Dropbox\22_PSQF6271\PSQF6271_Example6;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";
   
* Import chapter 6 stacked data and create time predictor variables;
DATA work.Example6; SET filesave.SAS_Chapter6; 
* Center time predictor for polynomial time models;
  time=session-1; LABEL time="time: Session (0=1)";
* Create session dummy codes for testing means model absolute fit in REML;
  IF session=1 THEN s1=1; ELSE s1=0;
  IF session=2 THEN s2=1; ELSE s2=0;
  IF session=3 THEN s3=1; ELSE s3=0;
  IF session=4 THEN s4=1; ELSE s4=0;
  IF session=5 THEN s5=1; ELSE s5=0;
  IF session=6 THEN s6=1; ELSE s6=0;
* Create predictors for piecewise time models assuming BALANCED DATA;
* slope12 and slope26 for 2 Direct Slopes model, time1 for Slope + Deviation Slope model;
* Both models have intercept at session 1, breakpoint at session 2;
       IF session=1 THEN DO; slope12=0; slope26=0; END;
  ELSE IF session=2 THEN DO; slope12=1; slope26=0; END;
  ELSE IF session=3 THEN DO; slope12=1; slope26=1; END;
  ELSE IF session=4 THEN DO; slope12=1; slope26=2; END;
  ELSE IF session=5 THEN DO; slope12=1; slope26=3; END;
  ELSE IF session=6 THEN DO; slope12=1; slope26=4; END;
  LABEL slope12="slope12: Early Practice Slope (Session 1-2)"
        slope26="slope26: Later Practice Slope (Session 2-6)";
* Example of piecewise predictors for UNBALANCED DATA (uses per-subject time variables instead);
*      IF session LE 1 THEN DO; * slope12=Time1-Time1; * slope26=Time2-Time2; * END;
* ELSE IF session LE 2 THEN DO; * slope12=Time2-Time1; * slope26=Time2-Time2; * END;
* ELSE IF session LE 3 THEN DO; * slope12=Time2-Time1; * slope26=Time3-Time2; * END;
* ELSE IF session LE 4 THEN DO; * slope12=Time2-Time1; * slope26=Time4-Time2; * END;
* ELSE IF session LE 5 THEN DO; * slope12=Time2-Time1; * slope26=Time5-Time2; * END;
* ELSE IF session LE 6 THEN DO; * slope12=Time2-Time1; * slope26=Time6-Time2; * END;
* LABEL 
* slope12 = "slope12: Early Practice Slope (Session 1-2)"
* slope26 = "slope26: Later Practice Slope (Session 2-6)"; 
* Log-transform time as new predictor;
  logtime=LOG(session); LABEL logtime="logtime: Log-Transformed Time (0=1)";
RUN;

***********************************************************************************;
*******                 DESCRIPTIVE AND POLYNOMIAL MODELS                   *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\SAS_Example6_Output_A.Polynomial.rtf" BODYTITLE STARTPAGE=NO STYLE=HTMLBlue;

TITLE1 "SAS Chapter 6 Example: Means by session for RT outcome";
* CLASS= means per session, WAYS= means overall=0 and per session=1;
PROC MEANS MEAN STDERR MIN MAX DATA=work.Example6; 
     CLASS session;
     WAYS 0 1;
     VAR rt;
RUN; TITLE1;

TITLE1 "SAS Ch 6: 0: Saturated Means, Unstructured Variance Model -- TOTAL ANSWER KEY";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = session / SOLUTION DDFM=Satterthwaite;
     REPEATED session / R RCORR TYPE=UN SUBJECT=PersonID;
     LSMEANS session / DIFF=ALL;
RUN; TITLE1; TITLE2;
 
TITLE1 "SAS Ch 6: 1b: Empty Means, Random Intercept Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovEmpty; * Save for pseudo-R2;
RUN; TITLE1;
 
TITLE1 "SAS Ch 6: 2a: Fixed Linear Time, Random Intercept Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovFixLin InfoCrit=FitFixLin; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from value of time predictor;
     ESTIMATE "Intercept at Session=1 Time=0"    intercept 1 time 0;
     ESTIMATE "Intercept at Session=2 Time=1"    intercept 1 time 1;
     ESTIMATE "Intercept at Session=3 Time=2"    intercept 1 time 2;
     ESTIMATE "Intercept at Session=4 Time=3"    intercept 1 time 3;
     ESTIMATE "Intercept at Session=5 Time=4"    intercept 1 time 4;
     ESTIMATE "Intercept at Session=6 Time=5"    intercept 1 time 5;
RUN; 
TITLE1 "Calculate pseudo R2 -- variance accounted for by fixed linear time";
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovFixLin); TITLE1;
 
TITLE1 "SAS Ch 6: 2b: Random Linear Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovRandLin InfoCrit=FitRandLin; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from value of time predictor;
     ESTIMATE "Intercept at Session=1 Time=0"    intercept 1 time 0;
     ESTIMATE "Intercept at Session=2 Time=1"    intercept 1 time 1;
     ESTIMATE "Intercept at Session=3 Time=2"    intercept 1 time 2;
     ESTIMATE "Intercept at Session=4 Time=3"    intercept 1 time 3;
     ESTIMATE "Intercept at Session=5 Time=4"    intercept 1 time 4;
     ESTIMATE "Intercept at Session=6 Time=5"    intercept 1 time 5;
RUN; 
TITLE1 "Calculate LRT -- does random linear time slope improve model fit?";
%FitTest(FitFewer=FitFixLin, FitMore=FitRandLin); TITLE1;
 
TITLE1 "SAS Ch 6: 3a: Fixed Quadratic, Random Linear Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovFixQuad InfoCrit=FitFixQuad; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from values of time predictors;
     ESTIMATE "Intercept at Session=1 Time=0"         intercept 1 time 0 time*time 0;
     ESTIMATE "Intercept at Session=2 Time=1"         intercept 1 time 1 time*time 1;
     ESTIMATE "Intercept at Session=3 Time=2"         intercept 1 time 2 time*time 4;
     ESTIMATE "Intercept at Session=4 Time=3"         intercept 1 time 3 time*time 9;
     ESTIMATE "Intercept at Session=5 Time=4"         intercept 1 time 4 time*time 16;
     ESTIMATE "Intercept at Session=6 Time=5"         intercept 1 time 5 time*time 25;
     * Get instantaneous linear slope per occasion from 2*value of time predictor;
     ESTIMATE "Linear Slope at Session=1 Time=0"      time 1 time*time 0;
     ESTIMATE "Linear Slope at Session=2 Time=1"      time 1 time*time 2;
     ESTIMATE "Linear Slope at Session=3 Time=2"      time 1 time*time 4;
     ESTIMATE "Linear Slope at Session=4 Time=3"      time 1 time*time 6;
     ESTIMATE "Linear Slope at Session=5 Time=4"      time 1 time*time 8;
     ESTIMATE "Linear Slope at Session=6 Time=5"      time 1 time*time 10;
RUN; 
TITLE1 "Calculate pseudo R2 -- variance accounted for by fixed quadratic time";
%PseudoR2(NCov=4, CovFewer=CovRandLin, CovMore=CovFixQuad); TITLE1;
 
TITLE1 "SAS Ch 6: 3b: Random Quadratic Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRandQuad; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors;
     ESTIMATE "Intercept at Session=1 Time=0"         intercept 1 time 0 time*time 0;
     ESTIMATE "Intercept at Session=2 Time=1"         intercept 1 time 1 time*time 1;
     ESTIMATE "Intercept at Session=3 Time=2"         intercept 1 time 2 time*time 4;
     ESTIMATE "Intercept at Session=4 Time=3"         intercept 1 time 3 time*time 9;
     ESTIMATE "Intercept at Session=5 Time=4"         intercept 1 time 4 time*time 16;
     ESTIMATE "Intercept at Session=6 Time=5"         intercept 1 time 5 time*time 25;
     * Get instantaneous linear slope per occasion from 2*value of time predictor;
     ESTIMATE "Linear Slope at Session=1 Time=0"      time 1 time*time 0;
     ESTIMATE "Linear Slope at Session=2 Time=1"      time 1 time*time 2;
     ESTIMATE "Linear Slope at Session=3 Time=2"      time 1 time*time 4;
     ESTIMATE "Linear Slope at Session=4 Time=3"      time 1 time*time 6;
     ESTIMATE "Linear Slope at Session=5 Time=4"      time 1 time*time 8;
     ESTIMATE "Linear Slope at Session=6 Time=5"      time 1 time*time 10;
RUN;
TITLE1 "Calculate LRT -- does random quadratic time slope improve model fit?";
%FitTest(FitFewer=FitFixQuad, FitMore=FitRandQuad); TITLE1;
 
TITLE1 "SAS: Test AR1 Residual Correlation in Random Quadratic Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID; 
     REPEATED session / R RCORR TYPE=AR(1) SUBJECT=PersonID; 
     ODS OUTPUT InfoCrit=FitRandQuadAR1; * Save for LRT;
RUN;
TITLE1 "Calculate LRT -- does AR1 residual correlation improve model fit?";
%FitTest(FitFewer=FitRandQuad, FitMore=FitRandQuadAR1); TITLE1;

TITLE1 "SAS: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)";
TITLE2 "Add 3 session dummy codes to saturate the means model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;  
     MODEL rt = time time*time s1 s2 s3 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     CONTRAST "Does fixed quadratic time reproduce saturated means?" s1 1, s2 1, s3 1 / CHISQ;
RUN; TITLE1; TITLE2;

TITLE1 "SAS: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)";
TITLE2 "Add different 3 session dummy codes to saturate the means model (same answer)";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;  
     MODEL rt = time time*time s4 s5 s6 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     CONTRAST "Does fixed quadratic time reproduce saturated means?" s4 1, s5 1, s6 1 / CHISQ;
RUN; TITLE1; TITLE2;

TITLE1 "SAS: Test Absolute Fit of Quadratic Time Variance Model (Using Fixed Quadratic Means Model)";
TITLE2 "Change to Unstructured R matrix as variance model answer key";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time time*time / SOLUTION DDFM=Satterthwaite;
     REPEATED session / R RCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitFixQuadUN; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- does random quadratic time reproduce UN variance model?";
%FitTest(FitFewer=FitRandQuad, FitMore=FitFixQuadUN); TITLE1;

* Close output;
ODS RTF CLOSE;

***********************************************************************************;
*******                        PIECEWISE MODELS                             *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\SAS_Example6_Output_B.Piecewise.rtf" BODYTITLE STARTPAGE=NO STYLE=HTMLBlue;

TITLE1 "SAS Ch 6: 4a: Fixed Slope12, Fixed Slope26, Random Intercept Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = slope12 slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovFix12Fix26 InfoCrit=FitFix12Fix26; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from values of time predictors, also slope difference;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 slope12 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 slope12 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 slope12 1 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 slope12 1 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 slope12 1 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 slope12 1 slope26 4;
     ESTIMATE "Difference of fixed slope12 vs slope26"     slope12 -1 slope26 1;
RUN;
TITLE1 "Calculate pseudo R2 -- variance accounted for by fixed slope12 and fixed slope26";
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovFix12Fix26); TITLE1;
 
TITLE1 "SAS Ch 6: 4b: Random Slope12, Fixed Slope26 Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = slope12 slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT slope12 / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRand12Fix26; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors, also slope difference;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 slope12 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 slope12 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 slope12 1 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 slope12 1 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 slope12 1 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 slope12 1 slope26 4;
     ESTIMATE "Difference of fixed slope12 vs slope26"     slope12 -1 slope26 1;
RUN; 
TITLE1 "Calculate LRT -- does random slope12 improve model fit?";
%FitTest(FitFewer=FitFix12Fix26, FitMore=FitRand12Fix26); TITLE1;
 
TITLE1 "SAS Ch 6: 4c: Random Slope12, Random Slope26 Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = slope12 slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT slope12 slope26 / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRand12Rand26; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors, also slope difference;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 slope12 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 slope12 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 slope12 1 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 slope12 1 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 slope12 1 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 slope12 1 slope26 4;
     ESTIMATE "Difference of fixed slope12 vs slope26"     slope12 -1 slope26 1;
RUN; 
TITLE1 "Calculate LRT -- does random slope26 improve model fit?";
%FitTest(FitFewer=FitRand12Fix26, FitMore=FitRand12Rand26); TITLE1;
 
TITLE1 "SAS Ch 6: 5a: Fixed Time, Fixed Slope26, Random Intercept Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitFix16Fix26; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors, also slope26;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 time 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 time 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 time 2 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 time 3 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 time 4 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 time 5 slope26 4;
     ESTIMATE "Fixed rate of change from session 2 to 6"   time 1 slope26 1;
RUN; TITLE1;
 
TITLE1 "SAS Ch 6: 5b: Random Time, Fixed Slope26 Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRand16Fix26; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors, also slope26;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 time 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 time 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 time 2 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 time 3 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 time 4 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 time 5 slope26 4;
     ESTIMATE "Fixed rate of change from session 2 to 6"   time 1 slope26 1;
RUN;
TITLE1 "Calculate LRT -- does random time slope improve model fit?";
%FitTest(FitFewer=FitFix16Fix26, FitMore=FitRand16Fix26); TITLE1;
 
TITLE1 "SAS Ch 6: 5c: Random Time, Random Slope26 Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time slope26 / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRand16Rand26; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors, also slope26;
     ESTIMATE "Intercept at Session=1 Time=0"              intercept 1 time 0 slope26 0;
     ESTIMATE "Intercept at Session=2 Time=1"              intercept 1 time 1 slope26 0;
     ESTIMATE "Intercept at Session=3 Time=2"              intercept 1 time 2 slope26 1;
     ESTIMATE "Intercept at Session=4 Time=3"              intercept 1 time 3 slope26 2;
     ESTIMATE "Intercept at Session=5 Time=4"              intercept 1 time 4 slope26 3;
     ESTIMATE "Intercept at Session=6 Time=5"              intercept 1 time 5 slope26 4;
     ESTIMATE "Fixed rate of change from session 2 to 6"   time 1 slope26 1;
RUN; 
TITLE1 "Calculate LRT -- does random slope26 improve model fit?";
%FitTest(FitFewer=FitRand16Fix26, FitMore=FitRand16Rand26); TITLE1;
 
TITLE1 "SAS: Test Absolute Fit of Two-Piece Means Model (Using Random Two-Piece Variance Model)";
TITLE2 "Add 3 session dummy codes to SECOND PIECE to saturate the means model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;  
     MODEL rt = time slope26 s4 s5 s6 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time slope26 / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     CONTRAST "Does fixed two-piece slope reproduce saturated means?" s4 1, s5 1, s6 1 / CHISQ;
RUN; TITLE1; TITLE2;

TITLE1 "SAS: Test Absolute Fit of Two-Piece Variance Model (Using Fixed Two-Piece Means Model)";
TITLE2 "Change to Unstructured R matrix as variance model answer key";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = time slope26 / SOLUTION DDFM=Satterthwaite;
     REPEATED session / R RCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitFixPieceUN; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- does random two-piece slope reproduce UN variance model?";
%FitTest(FitFewer=FitRand16Rand26, FitMore=FitFixPieceUN); TITLE1;

* Close external file of saved results;
ODS RTF CLOSE;

***********************************************************************************;
*******            TRULY EXPONENTIAL AND LATENT BASIS MODELS                *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\SAS_Example6_Output_C.Exponential_LatentBasis.rtf" BODYTITLE STARTPAGE=NO STYLE=HTMLBlue;

* Examples demonstrating how to use NLMIXED;

TITLE1 "SAS Model 1b: Empty Means, Random Intercept Model using ML";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST METHOD=ML;
     CLASS PersonID session;
     MODEL rt = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID; 
RUN; TITLE1;

TITLE1 "SAS Model 1b: Empty Means, Random Intercept Model via NLMIXED";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances and covariances;
  PARMS fint=1770  
        VarU0=198820 VarE=44900;
* Setting up level-2 equations;
  b0i = fint  + U0i; 
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i);
* Telling it which DV, defining level-1 residual;
* RT is normally distributed with a mean of "PredY" and a variance of "VarE";
  MODEL rt ~ normal(PredY, VarE);
* Random effects are normally distributed with means=0 and estimated variances;
  RANDOM  U0i ~ normal([0],[VarU0]) SUBJECT=PersonID;
* Asking for ICC and its SE;
  ESTIMATE "ICC" VarU0 / (VarU0 + VarE); 
RUN; TITLE1;

TITLE1 "SAS Model 3b: Random Quadratic Time Model using ML";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST METHOD=ML;
     CLASS PersonID session;
     MODEL rt = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID; 
RUN; TITLE1;

TITLE1 "SAS Model 3b: Random Quadratic Time Model via NLMIXED";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second and third lines are variances and covariances;
  PARMS fint=1946 flin=-121 fquad=14 
        VarU0=273306 CovU10=-35626 VarU1=25438 
        CovU20=3845 CovU21=-3838 VarU2=622 VarE=20298;
* Setting up level-2 equations;
  b0i = fint  + U0i; 
  b1i = flin  + U1i;
  b2i = fquad + U2i;
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i) + (b1i*time) + (b2i*time*time);
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "PredY" and a variance of "VarE";
  MODEL rt ~ normal(PredY, VarE);
* Random effects are normally distributed with means=0 and estimated variances/covariances;
  RANDOM U0i U1i U2i ~ normal([0,0,0],[VarU0,CovU10,VarU1,CovU20,CovU21,VarU2]) 
                       SUBJECT=PersonID; 
RUN; TITLE1;


TITLE1 "SAS Ch 6: 6a Fixed Asymptote Fixed Amount Fixed Rate Model";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances (in SD metric);
  PARMS fasymp= 1600 famount=300 frate=-1 
        SDE=600;
* Setting up level-2 equations;
  b0i = fasymp; 
  b1i = famount;
  b2i = frate;
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i) + (b1i*EXP(b2i*time));
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "PredY" and a variance of "VarE";
  MODEL rt ~ normal(PredY, sdE*sdE);
* Labeling estimated parameters;
  ESTIMATE "Fixed Asymptote"                    fasymp;
  ESTIMATE "Fixed Amount"                       famount;
  ESTIMATE "Fixed Rate"                         frate;
  ESTIMATE "Level-1 Residual E Variance"        sdE*sdE;
* Creating extra parameters and predicted means;
  ESTIMATE "Fixed Intercept"                    fasymp+famount;
  ESTIMATE "Session=1 Time=0 Predicted Mean"    fasymp+(famount*EXP(frate*0));
  ESTIMATE "Session=2 Time=1 Predicted Mean"    fasymp+(famount*EXP(frate*1));
  ESTIMATE "Session=3 Time=2 Predicted Mean"    fasymp+(famount*EXP(frate*2));
  ESTIMATE "Session=4 Time=3 Predicted Mean"    fasymp+(famount*EXP(frate*3));
  ESTIMATE "Session=5 Time=4 Predicted Mean"    fasymp+(famount*EXP(frate*4));
  ESTIMATE "Session=6 Time=5 Predicted Mean"    fasymp+(famount*EXP(frate*5));
RUN; TITLE1;

TITLE1 "SAS Ch 6: 6b Random Asymptote Fixed Amount Fixed Rate Model";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances (in SD metric);
  PARMS fasymp= 1675 famount=284 frate=-.7 
        sdE=474 sdU0=10; 
* Setting up level-2 equations;
  b0i = fasymp + U0i; 
  b1i = famount;
  b2i = frate;
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i) + (b1i*EXP(b2i*time));
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "PredY" and a variance of "VarE";
  MODEL rt ~ normal(PredY, sdE*sdE);
* Defining random effects: normally distributed with means and variances;
  RANDOM U0i ~ normal([0],[sdU0*sdU0]) SUBJECT=PersonID;
* Labeling estimated parameters;
  ESTIMATE "Fixed Asymptote"                        fasymp;
  ESTIMATE "Fixed Intercept"                        fasymp+famount;
  ESTIMATE "Fixed Amount"                           famount;
  ESTIMATE "Fixed Rate"                             frate;
  ESTIMATE "Level-1 Residual E Variance"            sdE*sdE;
  ESTIMATE "Level-2 Random Asymptote U0 Variance"   sdU0*sdU0;
* Creating extra parameters and predicted means;
  ESTIMATE "Fixed Intercept"                        fasymp+famount;
  ESTIMATE "Session=1 Time=0 Predicted Mean"        fasymp+(famount*EXP(frate*0));
  ESTIMATE "Session=2 Time=1 Predicted Mean"        fasymp+(famount*EXP(frate*1));
  ESTIMATE "Session=3 Time=2 Predicted Mean"        fasymp+(famount*EXP(frate*2));
  ESTIMATE "Session=4 Time=3 Predicted Mean"        fasymp+(famount*EXP(frate*3));
  ESTIMATE "Session=5 Time=4 Predicted Mean"        fasymp+(famount*EXP(frate*4));
  ESTIMATE "Session=6 Time=5 Predicted Mean"        fasymp+(famount*EXP(frate*5));
RUN; TITLE1;

TITLE1 "SAS Ch 6: 6c Random Asymptote Random Amount Fixed Rate Model";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances (in SD metric);
  PARMS fasymp=1675 famount=284 frate=-.7 
        sdE=184 sdU0=447 sdU01=1 sdU1=10;
* Setting up level-2 equations;
  b0i = fasymp  + U0i; 
  b1i = famount + U1i;
  b2i = frate;
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i) + (b1i*EXP(b2i*time));
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "y" and a variance of "VarE";
  MODEL rt ~ normal(PredY, sdE*sdE);
* Defining random effects: normally distributed with means and variances;
  RANDOM U0i U1i ~ normal([0,0],[sdU0*sdU0,sdU01*sdU01,sdU1*sdU1]) SUBJECT=PersonID;
* Labeling estimated parameters;
  ESTIMATE "Fixed Asymptote"                            fasymp;
  ESTIMATE "Fixed Amount"                               famount;
  ESTIMATE "Fixed Rate"                                 frate;
  ESTIMATE "Level-1 Residual E Variance"                sdE*sdE;
  ESTIMATE "Level-2 Random Asymptote U0 Variance"       sdU0*sdU0;
  ESTIMATE "Level-2 Asymptote-Amount U01 Covariance"    sdU01*sdU01;
  ESTIMATE "Level-2 Random Amount U1 Variance"          sdU1*sdU1;
  ESTIMATE "Level-2 Asymptote-Amount Correlation"       (sdU01*sdU01)/(sdU0*sdU1);
* Creating extra parameters and predicted means;
  ESTIMATE "Fixed Intercept"                            fasymp+famount;
  ESTIMATE "Session=1 Time=0 Predicted Mean"            fasymp+(famount*EXP(frate*0));
  ESTIMATE "Session=2 Time=1 Predicted Mean"            fasymp+(famount*EXP(frate*1));
  ESTIMATE "Session=3 Time=2 Predicted Mean"            fasymp+(famount*EXP(frate*2));
  ESTIMATE "Session=4 Time=3 Predicted Mean"            fasymp+(famount*EXP(frate*3));
  ESTIMATE "Session=5 Time=4 Predicted Mean"            fasymp+(famount*EXP(frate*4));
  ESTIMATE "Session=6 Time=5 Predicted Mean"            fasymp+(famount*EXP(frate*5));
RUN; TITLE1; 

TITLE1 "SAS Ch 6: 6d Random Asymptote Random Amount Random Rate Model";
TITLE2 "No convergence";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12; 
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances (in SD metric);
  PARMS fasymp=1678.25 famount=-282.72 frate=-.7323 
        sdE=130.88 sdU0=426.79 sdU01=87.68 sdU1=290.57
        sdU02=5.1343 sdU12=.08921 sdU2=1.3483;
* Setting up level-2 equations;
  b0i = fasymp  + U0i; 
  b1i = famount + U1i;
  b2i = frate   + U2i;
* Setting up level-1 equation WITHOUT level-1 residual;
  PredY = (b0i) + (b1i*EXP(b2i*time));
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "PredY" and a variance of "VarE";
MODEL rt ~ normal(PredY , sdE*sdE);
* Defining random effects: normally distributed with means and variances;
RANDOM  U0i U1i U2i ~ normal([0,0,0],[sdU0*sdU0,sdU01*sdU01,sdU1*sdU1,
                                     sdU02*sdU02,sdU12*sdU12,sdU2*sdU2]) SUBJECT=PersonID;
* Labeling estimated parameters;
  ESTIMATE "Fixed Asympote"                             fasymp;
  ESTIMATE "Fixed Amount"                               famount;
  ESTIMATE "Fixed Rate"                                 frate;
  ESTIMATE "Level-1 Residual E Variance"                sdE*sdE;
  ESTIMATE "Level-2 Random Asymptote U0 Variance"       sdU0*sdU0;
  ESTIMATE "Level-2 Asymptote-Amount U01 Covariance"    sdU01*sdU01;
  ESTIMATE "Level-2 Random Amount U1 Variance"          sdU1*sdU1;
  ESTIMATE "Level-2 Asymptote-Rate U02 Variance"        sdU02*sdU02;
  ESTIMATE "Level-2 Amount-Rate U12 Covariance"         sdU12*sdU12;
  ESTIMATE "Level-2 Random Rate U2 Variance"            sdU2*sdU2;
* Creating extra parameters and predicted means;
  ESTIMATE "Fixed Intercept"                            fasymp+famount;
  ESTIMATE "Session=1 Time=0 Predicted Mean"            fasymp+(famount*EXP(frate*0));
  ESTIMATE "Session=2 Time=1 Predicted Mean"            fasymp+(famount*EXP(frate*1));
  ESTIMATE "Session=3 Time=2 Predicted Mean"            fasymp+(famount*EXP(frate*2));
  ESTIMATE "Session=4 Time=3 Predicted Mean"            fasymp+(famount*EXP(frate*3));
  ESTIMATE "Session=5 Time=4 Predicted Mean"            fasymp+(famount*EXP(frate*4));
  ESTIMATE "Session=6 Time=5 Predicted Mean"            fasymp+(famount*EXP(frate*5));
RUN; TITLE1; TITLE2;

TITLE1 "SAS Test Absolute Fit of Means Model (Using Random Asymptote and Amount Variance Model)";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second line is variances (in SD metric);
  PARMS fasymp=1675 famount=284 frate=-.7 fs1=0 fs2=0 fs3=0
        sdE=184 sdU0=447 sdU01=1 sdU1=10;
* Setting up level-2 equations;
  b0i = fasymp  + U0i; 
  b1i = famount + U1i;
  b2i = frate;
* Setting up level-1 equation WITHOUT level-1 residual;
* Adding session-specific contrasts;
  PredY = (b0i) + (b1i*EXP(b2i*time)) + (fs1*s1) + (fs2*s2) + (fs3*s3);
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "y" and a variance of "VarE";
  MODEL rt ~ normal(PredY, sdE*sdE);
* Defining random effects: normally distributed with means and variances;
  RANDOM U0i U1i ~ normal([0,0],[sdU0*sdU0,sdU01*sdU01,sdU1*sdU1]) SUBJECT=PersonID;
* Multivariate Wald test for extra session contrasts;
  CONTRAST "Does fixed exponential reproduce saturated means?" fs1*1, fs2*1, fs3*1;
RUN; TITLE1; 

TITLE1 "SAS Ch 6: 0: Saturated Means, Unstructured Variance Model using ML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = session / SOLUTION CL CHISQ DDFM=Satterthwaite;
     REPEATED session / TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLSatUN; 
RUN; TITLE1;


TITLE1 "SAS Model 8: Random Latent Basis Model via NLMIXED (start values from Mplus)";
PROC NLMIXED DATA=work.Example6 METHOD=GAUSS TECH=NEWRAP GCONV=1e-12;
* Must define all parameters to be estimated and provide start values;
* First line is fixed effects, second and third lines are variances and covariances;
  PARMS fint=1965 fslp=-276 load1=.569 load2=.765 load3=.901 load4=.981
        VarU0=282599 CovU10=-83776 VarU1=76123 VarE=22936;
* Setting up level-2 equations;
  b0i = fint + U0i; 
  b1i = fslp + U1i;
* Setting up level-1 equation PER OCCASION WITHOUT level-1 residual;
       IF time=0 THEN PredY = (b0i) + (b1i*0);
  ELSE IF time=1 THEN PredY = (b0i) + (b1i*load1);
  ELSE IF time=2 THEN PredY = (b0i) + (b1i*load2);
  ELSE IF time=3 THEN PredY = (b0i) + (b1i*load3);
  ELSE IF time=4 THEN PredY = (b0i) + (b1i*load4);
  ELSE IF time=5 THEN PredY = (b0i) + (b1i*1);
* Telling it which DV, defining level-1 residual;
* RTs is normally distributed with a mean of "PredY" and a variance of "VarE";
  MODEL rt ~ normal(PredY, VarE);
* Random effects are normally distributed with means=0 and estimated variances/covariances;
  RANDOM U0i U1i ~ normal([0,0],[VarU0,CovU10,VarU1]) SUBJECT=PersonID; 
RUN; TITLE1;

* Close external file of saved results;
ODS RTF CLOSE;

***********************************************************************************;
*******             EXTRA LOG TIME MODELS FOR EXAMPLE 6                     *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\SAS_Example6_Output_D.LogTime.rtf" BODYTITLE STARTPAGE=NO STYLE=HTMLBlue;

TITLE1 "SAS: 7a: Fixed Linear Log Time, Random Intercept Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / G V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovFixLinLog InfoCrit=FitFixLinLog; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from value of time predictor;
     ESTIMATE "Intercept at Session=1"    intercept 1 logtime 0;
     ESTIMATE "Intercept at Session=2"    intercept 1 logtime 0.6931;
     ESTIMATE "Intercept at Session=3"    intercept 1 logtime 1.0986;
     ESTIMATE "Intercept at Session=4"    intercept 1 logtime 1.3863;
     ESTIMATE "Intercept at Session=5"    intercept 1 logtime 1.6094;
     ESTIMATE "Intercept at Session=6"    intercept 1 logtime 1.7918;
RUN; 
TITLE1 "Calculate pseudo R2 -- variance accounted for by fixed linear log time";
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovFixLinLog); TITLE1;

TITLE1 "SAS: 7b: Random Linear Log Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovRandLinLog  InfoCrit=FitRandLinLog; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from value of time predictor;
     ESTIMATE "Intercept at Session=1"    intercept 1 logtime 0;
     ESTIMATE "Intercept at Session=2"    intercept 1 logtime 0.6931;
     ESTIMATE "Intercept at Session=3"    intercept 1 logtime 1.0986;
     ESTIMATE "Intercept at Session=4"    intercept 1 logtime 1.3863;
     ESTIMATE "Intercept at Session=5"    intercept 1 logtime 1.6094;
     ESTIMATE "Intercept at Session=6"    intercept 1 logtime 1.7918;
RUN;
TITLE1 "Calculate LRT -- does random linear log time slope improve model fit?";
%FitTest(FitFewer=FitFixLinLog, FitMore=FitRandLinLog); TITLE1;

TITLE1 "SAS: Test Absolute Fit of Linear Log Time Means Model (Using Random Linear Log Time Variance Model)";
TITLE2 "Add 4 session dummy codes to saturate the means model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session; 
     MODEL rt = logtime s1 s2 s3 s4 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     CONTRAST "Does fixed linear log time reproduce saturated means?" s1 1, s2 1, s3 1, s4 1 / CHISQ;
RUN; TITLE1; TITLE2;

TITLE1 "SAS: Test Absolute Fit of Linear Log Time Variance Model (Using Fixed Linear Log Time Means Model)";
TITLE2 "Change to Unstructured R matrix as variance model answer key";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime / SOLUTION DDFM=Satterthwaite;
     REPEATED session / R RCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitFixLinLogUN; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- does random linear log time reproduce UN variance model?";
 %FitTest(FitFewer=FitRandLinLog, FitMore=FitFixLinLogUN); TITLE1;

TITLE1 "SAS: 7c: Fixed Quadratic, Random Linear Log Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime logtime*logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT CovParms=CovFixQuadLog InfoCrit=FitFixQuadLog; * Save for pseudo-R2 and LRT;
     * Get conditional mean per occasion from values of time predictors;
     ESTIMATE "Intercept at Session=1"    intercept 1 logtime 0      logtime*logtime 0;
     ESTIMATE "Intercept at Session=2"    intercept 1 logtime 0.6931 logtime*logtime 0.4805;
     ESTIMATE "Intercept at Session=3"    intercept 1 logtime 1.0986 logtime*logtime 1.2069;
     ESTIMATE "Intercept at Session=4"    intercept 1 logtime 1.3863 logtime*logtime 1.9218;
     ESTIMATE "Intercept at Session=5"    intercept 1 logtime 1.6094 logtime*logtime 2.5903;
     ESTIMATE "Intercept at Session=6"    intercept 1 logtime 1.7918 logtime*logtime 3.2104;
     * Get instantaneous linear slope per occasion from 2*value of time predictor;
     ESTIMATE "Linear Slope at Session=1" logtime 1 logtime*logtime 0;
     ESTIMATE "Linear Slope at Session=2" logtime 1 logtime*logtime 1.3863;
     ESTIMATE "Linear Slope at Session=3" logtime 1 logtime*logtime 2.1972;
     ESTIMATE "Linear Slope at Session=4" logtime 1 logtime*logtime 2.7726;
     ESTIMATE "Linear Slope at Session=5" logtime 1 logtime*logtime 3.2189;
     ESTIMATE "Linear Slope at Session=6" logtime 1 logtime*logtime 3.5835;
RUN; 
TITLE1 "Calculate pseudo R2 -- variance accounted for by fixed quadratic log time";
%PseudoR2(NCov=4, CovFewer=CovRandLinLog, CovMore=CovFixQuadLog); TITLE1;

TITLE1 "SAS: 7d: Random Quadratic Log Time Model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime logtime*logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime logtime*logtime / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitRandQuadLog; * Save for LRT;
     * Get conditional mean per occasion from values of time predictors;
     ESTIMATE "Intercept at Session=1"    intercept 1 logtime 0      logtime*logtime 0;
     ESTIMATE "Intercept at Session=2"    intercept 1 logtime 0.6931 logtime*logtime 0.4805;
     ESTIMATE "Intercept at Session=3"    intercept 1 logtime 1.0986 logtime*logtime 1.2069;
     ESTIMATE "Intercept at Session=4"    intercept 1 logtime 1.3863 logtime*logtime 1.9218;
     ESTIMATE "Intercept at Session=5"    intercept 1 logtime 1.6094 logtime*logtime 2.5903;
     ESTIMATE "Intercept at Session=6"    intercept 1 logtime 1.7918 logtime*logtime 3.2104;
     * Get instantaneous linear slope per occasion from 2*value of time predictor;
     ESTIMATE "Linear Slope at Session=1" logtime 1 logtime*logtime 0;
     ESTIMATE "Linear Slope at Session=2" logtime 1 logtime*logtime 1.3863;
     ESTIMATE "Linear Slope at Session=3" logtime 1 logtime*logtime 2.1972;
     ESTIMATE "Linear Slope at Session=4" logtime 1 logtime*logtime 2.7726;
     ESTIMATE "Linear Slope at Session=5" logtime 1 logtime*logtime 3.2189;
     ESTIMATE "Linear Slope at Session=6" logtime 1 logtime*logtime 3.5835;
RUN;
TITLE1 "Calculate LRT -- does random quadratic log time slope improve model fit?";
%FitTest(FitFewer=FitFixQuadLog, FitMore=FitRandQuadLog); TITLE1;

TITLE1 "SAS: Test Absolute Fit of Quadratic Log Time Means Model (Using Random Quadratic Log Time Variance Model)";
TITLE2 "Add 3 session dummy codes to saturate the means model";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session; 
     MODEL rt = logtime logtime*logtime s1 s2 s3 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime logtime*logtime / G GCORR V VCORR TYPE=UN SUBJECT=PersonID;
     REPEATED session / R TYPE=VC SUBJECT=PersonID;
     CONTRAST "Does fixed quadratic log time reproduce saturated means?" s1 1, s2 1, s3 1 / CHISQ;
RUN; TITLE1; TITLE2;

TITLE1 "SAS: Test Absolute Fit of Quadratic Log Time Variance Model (Using Fixed Quadratic Log Time Means Model)";
TITLE2 "Change to Unstructured R matrix as variance model answer key";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS PersonID session;
     MODEL rt = logtime logtime*logtime / SOLUTION DDFM=Satterthwaite;
     REPEATED session / R RCORR TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitFixQuadLogUN; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- does random quadratic log time reproduce UN variance model?";
%FitTest(FitFewer=FitRandQuadLog, FitMore=FitFixQuadLogUN);


***********************************************************************************;
*******     RE-RUNNING BEST VERSION OF EACH MODEL TO TEST ABSOLUTE FIT      *******;
***********************************************************************************;

TITLE1 "SAS Ch 6: 0: Saturated Means, Unstructured Variance Model";
TITLE2 "Using ML Instead of REML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = session / SOLUTION DDFM=Satterthwaite;
     REPEATED session / TYPE=UN SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLSatUN; * Save for LRT;
RUN; TITLE1; TITLE2;
 
TITLE1 "SAS Ch 6: 3b: Random Quadratic Time Model";
TITLE2 "Using ML instead of REML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = time1 time1*time1 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time1 time1*time1 / TYPE=UN SUBJECT=PersonID;
     REPEATED session / TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLRandQuad; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- test absolute fit of random quadratic time model";
%FitTest(FitFewer=FitMLRandQuad, FitMore=FitMLSatUN); TITLE1; 
 
TITLE1 "SAS Ch 6: 4c: Random Slope12, Random Slope26 Model";
TITLE2 "Using ML instead of REML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = slope12 slope26 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT slope12 slope26 / TYPE=UN SUBJECT=PersonID;
     REPEATED session / TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLRand12Rand26; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- test absolute fit of random two-piece model";
%FitTest(FitFewer=FitMLRand12Rand26, FitMore=FitMLSatUN); TITLE1;
 
TITLE1 "SAS Bonus: 7b: Random Linear Log Time Model";
TITLE2 "Using ML instead of REML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime / TYPE=UN SUBJECT=PersonID;
     REPEATED session / TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLRandLinLog; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- test absolute fit of random linear log time model";
%FitTest(FitFewer=FitMLRandLinLog, FitMore=FitMLSatUN); TITLE1;

TITLE1 "SAS Bonus: 7d: Random Quadratic Log Time Model";
TITLE2 "Using ML instead of REML";
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
     CLASS PersonID session;
     MODEL rt = logtime logtime*logtime / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT logtime logtime*logtime / TYPE=UN SUBJECT=PersonID;
     REPEATED session / TYPE=VC SUBJECT=PersonID;
     ODS OUTPUT InfoCrit=FitMLRandQuadLog; * Save for LRT;
RUN; TITLE2;
TITLE1 "Calculate LRT -- test absolute fit of random quadratic log time model";
%FitTest(FitFewer=FitMLRandQuadLog, FitMore=FitMLSatUN); TITLE1;



* Close external file of saved results;
ODS RTF CLOSE;


