// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

********************************************************************************
*******            BEGIN DATA MANIPULATION OF CHAPTER 6 EXAMPLE          *******
*******               CHANGE "filesave" to your directory                *******
********************************************************************************

// Define global variable for file location to be replaced in code below
   global filesave "C:\Dropbox\22_PSQF6271\PSQF6271_Example6"

// Import chapter 6 stacked data
   use "$filesave\STATA_Chapter6.dta", clear

// Center time predictor for polynomial time models (also need to make quadratic version) 
   gen time=session-1
   gen timesq=time*time
   label variable time   "time: Session (0=1)"
   label variable timesq "timesq: Quadratic Session (0=1)"
   
// Create session dummy codes for testing means model absolute fit in REML   
   gen s1=0
   gen s2=0
   gen s3=0
   gen s4=0
   gen s5=0
   gen s6=0
   replace s1=1 if session==1
   replace s2=1 if session==2
   replace s3=1 if session==3
   replace s4=1 if session==4
   replace s5=1 if session==5
   replace s6=1 if session==6
   
// Create predictors for piecewise time models assuming BALANCED DATA
// slope12 and slope26 for 2 Direct Slopes model, time for Slope + Deviation Slope model
// Both models have intercept at session 1, breakpoint at session 2
   gen slope12 = session
   recode slope12 (1=0) if session==1
   recode slope12 (2=1) if session==2
   recode slope12 (3=1) if session==3
   recode slope12 (4=1) if session==4
   recode slope12 (5=1) if session==5
   recode slope12 (6=1) if session==6
   gen slope26 = session
   recode slope26 (1=0) if session==1
   recode slope26 (2=0) if session==2
   recode slope26 (3=1) if session==3
   recode slope26 (4=2) if session==4
   recode slope26 (5=3) if session==5
   recode slope26 (6=4) if session==6
   label variable slope12 "slope12: Early Practice Slope (Session 1-2)"
   label variable slope26 "slope26: Later Practice Slope (Session 2-6)"
   
// Log-transform time as new predictor
   gen logtime=log(session)
   gen logtimesq=log(session)*log(session)
   label variable logtime   "logtime: Natural Log Session"
   label variable logtimesq "logtimesq: Quadratic Natural Log Session"
   

***********************************************************************************
*******                 DESCRIPTIVE AND POLYNOMIAL MODELS                   *******
***********************************************************************************

// Open external file to save results to
   log using $filesave\STATA_Example6_Output_A.Polynomial.log, replace name(Polynomial)

display "STATA Chapter 6 Example: Means by session for RT outcome"
tabulate session, summarize(rt)

display "STATA Ch 6: 0: Saturated Means, Unstructured Variance Model -- TOTAL ANSWER KEY"
mixed rt ib(last).session, || personid: , noconstant variance reml covariance(unstructured) ///
         residuals(unstructured,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2     // Print -2LL for model  
      estat ic, n(101)               // AIC and BIC
      estat wcorrelation, covariance // R matrix
      estat wcorrelation             // RCORR matrix
      contrast i.session, small      // Omnibus F-test for session 
      margins  i.session             // Means per session 
      margins  i.session, pwcompare(pveffects) df(100) // Mean diffs by session

display "STATA Ch 6: 1b: Empty Means, Random Intercept Model"
mixed rt , || personid: , variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2     // Print -2LL for model  
      estat ic, n(101)               // AIC and BIC
      estat icc                      // Intraclass correlation
      estat wcorrelation, covariance // V matrix
      estat wcorrelation             // VCORR matrix

display "STATA Ch 6: 2a: Fixed Linear Time, Random Intercept Model"
mixed rt c.time, || personid: , variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2     // Print -2LL for model  
      estat ic, n(101)               // AIC and BIC
      estat wcorrelation, covariance // V matrix
      estat wcorrelation             // VCORR matrix
       // Get conditional mean per occasion from value of time predictor
      lincom _cons*1 + c.time*0     // Intercept at Session=1 Time=0
      lincom _cons*1 + c.time*1     // Intercept at Session=2 Time=1
      lincom _cons*1 + c.time*2     // Intercept at Session=3 Time=2
      lincom _cons*1 + c.time*3     // Intercept at Session=4 Time=3
      lincom _cons*1 + c.time*4     // Intercept at Session=5 Time=4
      lincom _cons*1 + c.time*5     // Intercept at Session=6 Time=5
      estimates store FitFixLin     // Save for LRT

display "STATA Ch 6: 2b: Random Linear Time Model"
mixed rt c.time, || personid: time, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                         // Print -2LL for model  
      estat ic, n(101)                                   // AIC and BIC
      estat recovariance, relevel(personid)              // G matrix
      estat recovariance, relevel(personid) correlation  // GCORR matrix
      estat wcorrelation, covariance                     // V matrix
      estat wcorrelation                                 // VCORR matrix
       // Get conditional mean per occasion from value of time predictor
      lincom _cons*1 + c.time*0     // Intercept at Session=1 Time=0
      lincom _cons*1 + c.time*1     // Intercept at Session=2 Time=1
      lincom _cons*1 + c.time*2     // Intercept at Session=3 Time=2
      lincom _cons*1 + c.time*3     // Intercept at Session=4 Time=3
      lincom _cons*1 + c.time*4     // Intercept at Session=5 Time=4
      lincom _cons*1 + c.time*5     // Intercept at Session=6 Time=5
      estimates store FitRandLin    // Save for LRT
      lrtest FitRandLin FitFixLin   // Does random linear time slope improve model fit?

display "STATA Ch 6: 3a: Fixed Quadratic, Random Linear Time Model"
mixed rt c.time c.time#c.time, || personid: time, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model  
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors
      lincom _cons*1 + c.time*0 + c.time#c.time*0    // Intercept at Session=1 Time=0
      lincom _cons*1 + c.time*1 + c.time#c.time*1    // Intercept at Session=2 Time=1
      lincom _cons*1 + c.time*2 + c.time#c.time*4    // Intercept at Session=3 Time=2
      lincom _cons*1 + c.time*3 + c.time#c.time*9    // Intercept at Session=4 Time=3
      lincom _cons*1 + c.time*4 + c.time#c.time*16   // Intercept at Session=5 Time=4
      lincom _cons*1 + c.time*5 + c.time#c.time*25   // Intercept at Session=6 Time=5
       // Get instantaneous linear slope per occasion from 2*value of time predictor
      lincom c.time*1 + c.time#c.time*0,  small   // Linear Slope at Session=1 Time=0
      lincom c.time*1 + c.time#c.time*2,  small   // Linear Slope at Session=2 Time=1
      lincom c.time*1 + c.time#c.time*4,  small   // Linear Slope at Session=3 Time=2
      lincom c.time*1 + c.time#c.time*6,  small   // Linear Slope at Session=4 Time=3
      lincom c.time*1 + c.time#c.time*8,  small   // Linear Slope at Session=5 Time=4
      lincom c.time*1 + c.time#c.time*10, small   // Linear Slope at Session=6 Time=5
      estimates store FitFixQuad   // Save for LRT

display "STATA Ch 6: 3b: Random Quadratic Time Model"
mixed rt c.time c.time#c.time, || personid: time timesq, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model  
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors
      lincom _cons*1 + c.time*0 + c.time#c.time*0    // Intercept at Session=1 Time=0
      lincom _cons*1 + c.time*1 + c.time#c.time*1    // Intercept at Session=2 Time=1
      lincom _cons*1 + c.time*2 + c.time#c.time*4    // Intercept at Session=3 Time=2
      lincom _cons*1 + c.time*3 + c.time#c.time*9    // Intercept at Session=4 Time=3
      lincom _cons*1 + c.time*4 + c.time#c.time*16   // Intercept at Session=5 Time=4
      lincom _cons*1 + c.time*5 + c.time#c.time*25   // Intercept at Session=6 Time=5
       // Get instantaneous linear slope per occasion from 2*value of time predictor
      lincom c.time*1 + c.time#c.time*0,  small   // Linear Slope at Session=1 Time=0
      lincom c.time*1 + c.time#c.time*2,  small   // Linear Slope at Session=2 Time=1
      lincom c.time*1 + c.time#c.time*4,  small   // Linear Slope at Session=3 Time=2
      lincom c.time*1 + c.time#c.time*6,  small   // Linear Slope at Session=4 Time=3
      lincom c.time*1 + c.time#c.time*8,  small   // Linear Slope at Session=5 Time=4
      lincom c.time*1 + c.time#c.time*10, small   // Linear Slope at Session=6 Time=5
      estimates store FitRandQuad     // Save for LRT
      lrtest FitRandQuad FitFixQuad   // Does random quadratic time slope improve fit? 

display "STATA: Test AR1 Residual Correlation in Random Quadratic Time Model"
mixed rt c.time c.time#c.time, || personid: time timesq, variance reml covariance(unstructured) ///
         residuals(ar1,t(session)) dfmethod(satterthwaite) dftable(pvalue)   
      display "-2LL = " e(ll)*-2         // Print -2LL for model  
      estimates store FitRandQuadAR1     // Save for LRT
      lrtest FitRandQuadAR1 FitFixQuad   // Does AR1 residual correlation improve model fit?           

display "STATA: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)"
display "Add 3 session dummy codes to saturate the means model"
mixed rt c.time c.time#c.time c.s1 c.s2 c.s3, ///
         || personid: time timesq, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      test (c.s1=0)(c.s2=0)(c.s3=0), small  // Does fixed quadratic time reproduce saturated means?
           
display "STATA: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)"
display "Add different 3 session dummy codes to saturate the means model"
mixed rt c.time c.time#c.time c.s4 c.s5 c.s6, ///
         || personid: time timesq, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2            // Print -2LL for model  
      test (c.s4=0)(c.s5=0)(c.s6=0), small  // Does fixed quadratic time reproduce saturated means?
       
display "STATA: Test Absolute Fit of Quadratic Time Variance Model (Using Fixed Quadratic Means Model)"
display "Change to Unstructured R matrix as variance model answer key"
mixed rt c.time c.time#c.time, || personid: , noconstant variance reml ///
         residuals(unstructured,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2       // Print -2LL for model  
      estimates store FitFixQuadUN     // Save for LRT
      lrtest FitFixQuadUN FitRandQuad  // Does random quadratic time reproduce UN variance model?

// Close external file of saved results
   log close Polynomial
   
   
***********************************************************************************;
*******                        PIECEWISE MODELS                             *******;
***********************************************************************************;

// Open external file to save results to
   log using $filesave\STATA_Example6_Output_B.Piecewise.log, replace name(Piecewise)

display "STATA Ch 6: 4a: Fixed Slope12, Fixed Slope26, Random Intercept Model"
mixed rt c.slope12 c.slope26, || personid: , variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                   // Print -2LL for model  
      estat ic, n(101)                             // AIC and BIC
      estat wcorrelation, covariance               // V matrix
      estat wcorrelation                           // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope difference
      lincom _cons*1 + c.slope12*0 + c.slope26*0   // Intercept at Session=1
      lincom _cons*1 + c.slope12*1 + c.slope26*0   // Intercept at Session=2
      lincom _cons*1 + c.slope12*1 + c.slope26*1   // Intercept at Session=3 
      lincom _cons*1 + c.slope12*1 + c.slope26*2   // Intercept at Session=4
      lincom _cons*1 + c.slope12*1 + c.slope26*3   // Intercept at Session=5 
      lincom _cons*1 + c.slope12*1 + c.slope26*4   // Intercept at Session=6
      lincom c.slope12*-1 + c.slope26*1            // Difference of fixed slope12 vs slope26
      estimates store FitFix12Fix26                // Save for LRT

display "STATA Ch 6: 4b: Random Slope12, Fixed Slope26 Model"
mixed rt c.slope12 c.slope26, || personid: slope12, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope difference
      lincom _cons*1 + c.slope12*0 + c.slope26*0   // Intercept at Session=1
      lincom _cons*1 + c.slope12*1 + c.slope26*0   // Intercept at Session=2
      lincom _cons*1 + c.slope12*1 + c.slope26*1   // Intercept at Session=3
      lincom _cons*1 + c.slope12*1 + c.slope26*2   // Intercept at Session=4
      lincom _cons*1 + c.slope12*1 + c.slope26*3   // Intercept at Session=5 
      lincom _cons*1 + c.slope12*1 + c.slope26*4   // Intercept at Session=6
      lincom c.slope12*-1 + c.slope26*1, small     // Difference of fixed slope12 vs slope26
      estimates store FitRand12Fix26        // Save for LRT
      lrtest FitRand12Fix26 FitFix12Fix26   // Does random slope12 improve model fit?

display "STATA Ch 6: 4c: Random Slope12, Random Slope26 Model"
mixed rt c.slope12 c.slope26, || personid: slope12 slope26, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope difference
      lincom _cons*1 + c.slope12*0 + c.slope26*0   // Intercept at Session=1
      lincom _cons*1 + c.slope12*1 + c.slope26*0   // Intercept at Session=2
      lincom _cons*1 + c.slope12*1 + c.slope26*1   // Intercept at Session=3 
      lincom _cons*1 + c.slope12*1 + c.slope26*2   // Intercept at Session=4
      lincom _cons*1 + c.slope12*1 + c.slope26*3   // Intercept at Session=5 
      lincom _cons*1 + c.slope12*1 + c.slope26*4   // Intercept at Session=6
      lincom c.slope12*-1 + c.slope26*1, small     // Difference of fixed slope12 vs slope26
      estimates store FitRand12Rand26              // Save for LRT
      lrtest FitRand12Rand26 FitRand12Fix26        // Does random slope26 improve model fit?

display "STATA Ch 6: 5a: Fixed Time, Fixed Slope26, Random Intercept Model"
mixed rt c.time c.slope26, || personid: , variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                   // Print -2LL for model 
      estat ic, n(101)                             // AIC and BIC
      estat wcorrelation, covariance               // V matrix
      estat wcorrelation                           // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope26
      lincom _cons*1 + c.time*0 + c.slope26*0     // Intercept at Session=1
      lincom _cons*1 + c.time*1 + c.slope26*0     // Intercept at Session=2
      lincom _cons*1 + c.time*1 + c.slope26*1     // Intercept at Session=3 
      lincom _cons*1 + c.time*1 + c.slope26*2     // Intercept at Session=4
      lincom _cons*1 + c.time*1 + c.slope26*3     // Intercept at Session=5 
      lincom _cons*1 + c.time*1 + c.slope26*4     // Intercept at Session=6
      lincom c.time*1 + c.slope26*1, small        // Fixed rate of change from session 2 to 6
      estimates store FitFix16Fix26               // Save for LRT

display "STATA Ch 6: 5b: Random Time, Fixed Slope26 Model"
mixed rt c.time c.slope26, || personid: time, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope26
      lincom _cons*1 + c.time*0 + c.slope26*0     // Intercept at Session=1
      lincom _cons*1 + c.time*1 + c.slope26*0     // Intercept at Session=2
      lincom _cons*1 + c.time*2 + c.slope26*1     // Intercept at Session=3
      lincom _cons*1 + c.time*3 + c.slope26*2     // Intercept at Session=4
      lincom _cons*1 + c.time*4 + c.slope26*3     // Intercept at Session=5
      lincom _cons*1 + c.time*5 + c.slope26*4     // Intercept at Session=6
      lincom c.time*1 + c.slope26*1, small        // Fixed rate of change from session 2 to 6
      estimates store FitRand16Fix26              // Save for LRT
      lrtest FitRand16Fix26 FitFix16Fix26         // Does random time slope improve model fit?

display "STATA Ch 6: 5c: Random Time, Random Slope26 Model"
mixed rt c.time c.slope26, || personid: time slope26, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional mean per occasion from values of time predictors, also slope26
      lincom _cons*1 + c.time*0 + c.slope26*0     // Intercept at Session=1
      lincom _cons*1 + c.time*1 + c.slope26*0     // Intercept at Session=2
      lincom _cons*1 + c.time*2 + c.slope26*1     // Intercept at Session=3
      lincom _cons*1 + c.time*3 + c.slope26*2     // Intercept at Session=4
      lincom _cons*1 + c.time*4 + c.slope26*3     // Intercept at Session=5
      lincom _cons*1 + c.time*5 + c.slope26*4     // Intercept at Session=6
      lincom c.time*1 + c.slope26*1, small        // Fixed rate of change from session 2 to 6
      estimates store FitRand16Rand26             // Save for LRT
      lrtest FitRand16Rand26 FitRand16Fix26       // Does random slope26 improve model fit?

display "STATA: Test Absolute Fit of Two-Piece Means Model (Using Random Two-Piece Variance Model)"
display "Add 3 session dummy codes to SECOND PIECE to saturate the means model"
mixed rt c.time c.slope26 c.s4 c.s5 c.s6, ///
         || personid: time slope26, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2            // Print -2LL for model 
      test (c.s4=0)(c.s5=0)(c.s6=0), small  // Does fixed two-piece slope reproduce saturated means?
                 
display "STATA: Test Absolute Fit of Two-Piece Variance Model (Using Fixed Two-Piece Means Model)"
display "Change to Unstructured R matrix as variance model answer key"
mixed rt c.time c.slope26, || personid: , noconstant variance reml ///
         residuals(unstructured,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2            // Print -2LL for model 
      estimates store FitFixPieceUN         // Save for LRT
      lrtest FitFixPieceUN FitRand16Rand26  // Does random two-piece slope reproduce UN variance model?
       
// Close external file of saved results
   log close Piecewise
   

***********************************************************************************
*******             EXTRA LOG TIME MODELS FOR EXAMPLE 6                     *******
***********************************************************************************    
       
// Open external file to save results to
   log using $filesave\STATA_Example6_Output_D.LogTime.log, replace name(LogTime)     
   
display "STATA: 7a: Fixed Linear Log Time, Random Intercept Model"
mixed rt c.logtime, || personid: , variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2     // Print -2LL for model 
      estat ic, n(101)               // AIC and BIC
      estat wcorrelation, covariance // V matrix
      estat wcorrelation             // VCORR matrix
       // Get conditional means per occasion from values of time predictor
      lincom _cons*1 + c.logtime*0          // Intercept at Session=1 
      lincom _cons*1 + c.logtime*0.6931     // Intercept at Session=2 
      lincom _cons*1 + c.logtime*1.0986     // Intercept at Session=3 
      lincom _cons*1 + c.logtime*1.3863     // Intercept at Session=4 
      lincom _cons*1 + c.logtime*1.6094     // Intercept at Session=5 
      lincom _cons*1 + c.logtime*1.7918     // Intercept at Session=6 
      estimates store FitFixLinLog          // Save for LRT

display "STATA: 7b: Random Linear Log Time Model"
mixed rt c.logtime,  || personid: logtime, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional means per occasion from values of time predictor
      lincom _cons*1 + c.logtime*0          // Intercept at Session=1 
      lincom _cons*1 + c.logtime*0.6931     // Intercept at Session=2 
      lincom _cons*1 + c.logtime*1.0986     // Intercept at Session=3 
      lincom _cons*1 + c.logtime*1.3863     // Intercept at Session=4 
      lincom _cons*1 + c.logtime*1.6094     // Intercept at Session=5 
      lincom _cons*1 + c.logtime*1.7918     // Intercept at Session=6 
      estimates store FitRandLinLog         // Save for LRT
      lrtest FitRandLinLog FitFixLinLog     // Does random linear log time slope improve model fit?

display "STATA: Test Absolute Fit of the Means Model (Using Random Linear Log Time Variance Model)"
display "Add 4 session dummy codes to saturate the means model"
mixed rt c.logtime c.s1 c.s2 c.s3 c.s4, ///
         || personid: logtime, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                    // Print -2LL for model 
      test (c.s1=0)(c.s2=0)(c.s3=0)(c.s4=0), small  // Does fixed linear log time reproduce saturated means?
                 
display "STATA: Test Absolute Fit of the Variance Model (Using Fixed Linear Log Time Means Model)"
mixed rt c.logtime, || personid: , noconstant variance reml ///
         residuals(unstructured,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      estimates store FitFixLinLogUN      // Save for LRT
      lrtest FitFixLinLogUN FitRandLinLog // Does random linear log time reproduce UN variance model?     
       
display "STATA: 7c: Fixed Quadratic, Random Linear Log Time Model"
mixed rt c.logtime c.logtime#c.logtime, || personid: logtime, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional means per occasion from values of time predictors
      lincom _cons*1 + c.logtime*0      + c.logtime#c.logtime*0        // Intercept at Session=1
      lincom _cons*1 + c.logtime*0.6931 + c.logtime#c.logtime*0.4805   // Intercept at Session=2
      lincom _cons*1 + c.logtime*1.0986 + c.logtime#c.logtime*1.2069   // Intercept at Session=3
      lincom _cons*1 + c.logtime*1.3863 + c.logtime#c.logtime*1.9218   // Intercept at Session=4
      lincom _cons*1 + c.logtime*1.6094 + c.logtime#c.logtime*2.5903   // Intercept at Session=5
      lincom _cons*1 + c.logtime*1.7918 + c.logtime#c.logtime*3.2104   // Intercept at Session=6
       // Get instantaneous linear slope per occasion from 2*value of time predictor
       lincom c.logtime*1 + c.logtime#c.logtime*0     , small    // Linear Slope at Session=1
       lincom c.logtime*1 + c.logtime#c.logtime*1.3863, small    // Linear Slope at Session=2
       lincom c.logtime*1 + c.logtime#c.logtime*2.1972, small    // Linear Slope at Session=3
       lincom c.logtime*1 + c.logtime#c.logtime*2.7726, small    // Linear Slope at Session=4
       lincom c.logtime*1 + c.logtime#c.logtime*3.2189, small    // Linear Slope at Session=5
       lincom c.logtime*1 + c.logtime#c.logtime*3.5835, small    // Linear Slope at Session=6
       estimates store FitFixQuadLog                             // Save for LRT

display "STATA: 7d: Random Quadratic Log Time Model"
mixed rt c.logtime c.logtime#c.logtime, || personid: logtime logtimesq, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2                        // Print -2LL for model 
      estat ic, n(101)                                  // AIC and BIC
      estat recovariance, relevel(personid)             // G matrix
      estat recovariance, relevel(personid) correlation // GCORR matrix
      estat wcorrelation, covariance                    // V matrix
      estat wcorrelation                                // VCORR matrix
       // Get conditional means per occasion from values of time predictors
      lincom _cons*1 + c.logtime*0      + c.logtime#c.logtime*0        // Intercept at Session=1
      lincom _cons*1 + c.logtime*0.6931 + c.logtime#c.logtime*0.4805   // Intercept at Session=2
      lincom _cons*1 + c.logtime*1.0986 + c.logtime#c.logtime*1.2069   // Intercept at Session=3
      lincom _cons*1 + c.logtime*1.3863 + c.logtime#c.logtime*1.9218   // Intercept at Session=4
      lincom _cons*1 + c.logtime*1.6094 + c.logtime#c.logtime*2.5903   // Intercept at Session=5
      lincom _cons*1 + c.logtime*1.7918 + c.logtime#c.logtime*3.2104   // Intercept at Session=6
      // Get instantaneous linear slope per occasion from 2*value of time predictor
      lincom c.logtime*1 + c.logtime#c.logtime*0     , small    // Linear Slope at Session=1
      lincom c.logtime*1 + c.logtime#c.logtime*1.3863, small    // Linear Slope at Session=2
      lincom c.logtime*1 + c.logtime#c.logtime*2.1972, small    // Linear Slope at Session=3
      lincom c.logtime*1 + c.logtime#c.logtime*2.7726, small    // Linear Slope at Session=4
      lincom c.logtime*1 + c.logtime#c.logtime*3.2189, small    // Linear Slope at Session=5
      lincom c.logtime*1 + c.logtime#c.logtime*3.5835, small    // Linear Slope at Session=6
      estimates store FitRandQuadLog        // Save for LRT
      lrtest FitRandQuadLog FitFixQuadLog   // Does random quadratic log time slope improve model fit? 

display "STATA: Test Absolute Fit of Quadratic Log Time Means Model (Using Random Quadratic Log Time Variance Model)"
display "Add 3 session dummy codes to saturate the means model"
mixed rt c.logtime c.logtime#c.logtime c.s1 c.s2 c.s3, ///
         || personid: logtime logtimesq, variance reml covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2            // Print -2LL for model 
      test (c.s1=0)(c.s2=0)(c.s3=0), small  // Does fixed quadratic log time reproduce saturated means?
                 
display "STATA: Test Absolute Fit of Quadratic Log Time Variance Model (Using Fixed Quadratic Log Time Means Model)"
display "Change to Unstructured R matrix as variance model answer key"
mixed rt c.logtime c.logtime#c.logtime, || personid: , noconstant variance reml ///
         residuals(unstructured,t(session)) dfmethod(satterthwaite) dftable(pvalue)
      display "-2LL = " e(ll)*-2            // Print -2LL for model 
      estimates store FitFixQuadLogUN       // Save for LRT
      lrtest FitFixQuadLogUN FitRandQuadLog // Does random quadratic log time reproduce UN variance model?
   
   
***********************************************************************************
*******     RE-RUNNING BEST VERSION OF EACH MODEL TO TEST ABSOLUTE FIT      *******
***********************************************************************************

display "STATA Ch 6: 0: Saturated Means, Unstructured Variance Model"
display "Using ML Instead of REML (so residual DF instead of Satterthwaite)"
mixed rt i.session, || personid: , noconstant variance mle covariance(unstructured) ///
         residuals(unstructured,t(session)) dfmethod(residual) dftable(pvalue)
      display "-2LL = " e(ll)*-2    // Print -2LL for model 
      estimates store FitMLSatUN    // Save for LRT

display "STATA Ch 6: 3b: Random Quadratic Time Model"
display "Using ML Instead of REML (so residual DF instead of Satterthwaite)"
mixed rt c.time c.time#c.time, || personid: time timesq, variance mle covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(residual) dftable(pvalue)
      display "-2LL = " e(ll)*-2        // Print -2LL for model 
      estimates store FitMLRandQuad     // Save for LRT
      lrtest FitMLSatUN FitMLRandQuad   // Test absolute fit of random quadratic time model 

display "STATA Ch 6: 4c: Random Slope12, Random Slope26 Model"
display "Using ML Instead of REML (so residual DF instead of Satterthwaite)"
mixed rt c.slope12 c.slope26, || personid: slope12 slope26, variance mle covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(residual) dftable(pvalue)
      display "-2LL = " e(ll)*-2           // Print -2LL for model 
      estimates store FitMLRand12Rand26    // Save for LRT
      lrtest FitMLSatUN FitMLRand12Rand26  // Test absolute fit of random two-piece model
       
display "STATA: 7b: Random Linear Log Time Model"
display "Using ML Instead of REML (so residual DF instead of Satterthwaite)"
mixed rt c.logtime,  || personid: logtime, variance mle covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(residual) dftable(pvalue)
      display "-2LL = " e(ll)*-2          // Print -2LL for model 
      estimates store FitMLRandLinLog     // Save for LRT
      lrtest FitMLSatUN FitMLRandLinLog   // Test absolute fit of random linear log time model
       
display "STATA: 7d: Random Quadratic Log Time Model"
display "Using ML Instead of REML (so residual DF instead of Satterthwaite)"
mixed rt c.logtime c.logtime#c.logtime, || personid: logtime logtimesq, variance mle covariance(unstructured) ///
         residuals(independent,t(session)) dfmethod(residual) dftable(pvalue)
      display "-2LL = " e(ll)*-2           // Print -2LL for model 
      estimates store FitMLRandQuadLog     // Save for LRT
      lrtest FitMLSatUN FitMLRandQuadLog   // Test absolute fit of random quadratic log time model
       
// Close external file of saved results
   log close LogTime
